/*
 * Title   : example-test1
 * Author  : Tim Brecht
 * Date    : Sat Oct  2 08:19:57 EST 1999
 *
 * Some example tests using Create, Open, and Close.
 * Assumes that files named FILE1 and FILE2 do not exist in current directory
 *
 * Modified: Thu Dec 23 17:05:19 EST 2004
 *   TBB - updated for Winter 2005 term.
 */

#include "testutils.h"
#include "syscall.h"

int
main()
{
  OpenFileId f1, f2;
  int i = 42;
  int j = -999;
  int intbuf = 0;
  int rc = 0;      /* return code */

  /* Useful for debugging but once tests work don't want verbose output */
  /*
  TEST_VERBOSE_ON();
  */

  /* Check that create succeeds */
  rc = Create("FILE1");
  /* returns 0 on success */
  TEST_EQUAL(rc, SUCCESS, "Unable to create FILE1");

  /* Check that create succeeds */
  rc = Create("FILE2");
  /* returns 0 on success */
  TEST_EQUAL(rc, SUCCESS, "Unable to create FILE2");

  /* Check that open succeeds */
  f1 = Open("FILE1", "rw");
  TEST_POSITIVE(f1, "Unable to open FILE1");

  /* Check that open succeeds */
  f2 = Open("FILE2", "rw");
  TEST_POSITIVE(f2, "Unable to open FILE2");

  TEST_NOT_EQUAL(f1, f2, "OpenFileId f1 == f2");

  /* Write something simple to file 1 */
  rc = Write((char *) &i, sizeof(i), f1);
  TEST_EQUAL(rc, sizeof(i), "Write to f1 does not write/return proper value");

  /* Write something simple to file 2 */
  rc = Write((char *) &j, sizeof(j), f2);
  TEST_EQUAL(rc, sizeof(j), "Write to f2 does not write/return proper value");

  rc = Close(f1);
  TEST_EQUAL(rc, SUCCESS, "Close f1 failed");

  rc = Close(f1);
  /* closing a second time should fail - it's already closed */
  TEST_NEGATIVE(rc, "Close f1 second time didn't fail");

  rc = Close(f2);
  TEST_EQUAL(rc, SUCCESS, "Close f2 failed");

  f1 = Open("FILE1", "rw");
  TEST_POSITIVE(f1, "Unable to open FILE1, after Close");

  f2 = Open("FILE2", "rw");
  TEST_POSITIVE(f1, "Unable to open FILE2, after Close");

  TEST_NOT_EQUAL(f1, f2, "OpenFileId f1 == f2");

  rc = Read((char *) &intbuf, sizeof(intbuf), f1);
  TEST_EQUAL(rc, sizeof(intbuf), 
     "Read from f1 does not read/return proper value");
  TEST_EQUAL(intbuf, i,
     "Read from f1 did not get correct value");

  rc = Read((char *) &intbuf, sizeof(intbuf), f2);
  TEST_EQUAL(rc, sizeof(j), "Read from f2 does not read/return proper value");
  TEST_EQUAL(intbuf, j, "Read from f2 did not get correct value");

  TEST_STATS();

  Halt();
}

